// Demonstrates how to use parallel for loop
// (c) Stefan Zellmann 2018

#include <cstddef>
#include <thread>

#include "parallel_for.h"

int main()
{
    using namespace visionaray;

    // Keep a thread pool active all the time
    thread_pool pool(std::thread::hardware_concurrency());


// 1.) Simple loop

    // Serial
    for (int i = 0; i < 1000; ++i)
    {
        // use i
    }

    // Parallel
    parallel_for(pool, range1d<int>(0, 1000), [](int i)
    {
        // use i
    });


// 2.) Simple loop, modify data from scope above

    // Serial
    constexpr long N = 100;
    long arr[N];
    for (long i = 0; i < N; ++i)
    {
        arr[i] = i;
    }

    // Parallel
    parallel_for(pool, range1d<long>(0, N), [&arr](long i)
    {
        // Note how we capture arr by reference above
        // so we can modify it from inside the lambda
        arr[i] = i;
    });


// 3.) Loop with stride != 1

    // Serial
    std::size_t first =  0;
    std::size_t last  =  32;
    std::size_t stride = 4;

    for (std::size_t i = first; i < last; i += stride)
    {
        std::size_t first = i;
        std::size_t last  = i + stride;
    }

    parallel_for(pool, tiled_range1d<std::size_t>(first, last, stride),
        [](range1d<std::size_t> const& r)
    {
        std::size_t first = r.begin();
        std::size_t last  = r.end();
    });
}
